#pragma once

#include "CoreMinimal.h"
#include "GameFramework/Volume.h"
#include "Misc/MujocoOptions.h"
#include "Components/BillboardComponent.h"
#include "Misc/MujocoWrapper.h"
#include "Delegates/DelegateCombinations.h"
#include "MujocoVolumeActor.generated.h"

class UMujocoBodyComponent;
class UMujocoJointComponent;
class UMujocoGeomComponent;
class UMujocoSiteComponent;
class UMujocoActuatorComponent;
class UMujocoEqualityComponent;
class UMujocoTendonComponent;


DECLARE_DYNAMIC_MULTICAST_DELEGATE(FOnMujocoCompileBegin);
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FOnMujocoCompileError, FString, Error);
DECLARE_DYNAMIC_MULTICAST_DELEGATE(FOnMujocoCompileSuccess);

UCLASS(Blueprintable, BlueprintType)
class LUCKYMUJOCO_API AMujocoVolumeActor : public AActor
{
	GENERATED_BODY()

	TMujocoModelPtr MujocoModel;
	TMujocoDataPtr MujocoData;

	UPROPERTY(Transient, VisibleAnywhere, Category = "Mujoco | Debug")
	TArray<TSoftObjectPtr<UMujocoBodyComponent>> BodyComponents;

	UPROPERTY(Transient, VisibleAnywhere, Category = "Mujoco | Debug")
	TArray<TSoftObjectPtr<UMujocoGeomComponent>> GeomComponents;

	UPROPERTY(Transient, VisibleAnywhere, Category = "Mujoco | Debug")
	TArray<TSoftObjectPtr<UMujocoSiteComponent>> SiteComponents;

	UPROPERTY(Transient, VisibleAnywhere, Category = "Mujoco | Debug")
	TArray<TSoftObjectPtr<UMujocoJointComponent>> JointComponents;

	UPROPERTY(Transient, VisibleAnywhere, Category = "Mujoco | Debug")
	TArray<TSoftObjectPtr<UMujocoActuatorComponent>> ActuatorComponents;

	UPROPERTY(Transient, VisibleAnywhere, Category = "Mujoco | Debug")
	TArray<TSoftObjectPtr<UMujocoEqualityComponent>> EqualityComponents;

	UPROPERTY(Transient, VisibleAnywhere, Category = "Mujoco | Debug")
	TArray<TSoftObjectPtr<UMujocoTendonComponent>> TendonComponents;	

	UFUNCTION(BlueprintCallable, Category = "Mujoco")
	void InitializeMujoco();

	template <typename ComponentType> void AssignComponentsToArray(UWorld* World, TArray<TSoftObjectPtr<ComponentType>>& ComponentArray);

public:
	AMujocoVolumeActor();	

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Simulation", meta = (Min = 0, Max = 100, ClampMin = 0, ClampMax = 100))
	int32 FrameSkip = 0;

	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Category = "Mujoco")
	TObjectPtr<UBillboardComponent> SpriteComponent;

	UPROPERTY(VisibleInstanceOnly, NoClear, Instanced, Category = "Mujoco")
	TObjectPtr<UMujocoExportOptions> Options;	

	UPROPERTY(BlueprintAssignable, Category = "Mujoco | Events")
	FOnMujocoCompileBegin OnMujocoCompileBegin;

	UPROPERTY(BlueprintAssignable, Category = "Mujoco | Events")
	FOnMujocoCompileError OnMujocoCompileError;

	UPROPERTY(BlueprintAssignable, Category = "Mujoco | Events")
	FOnMujocoCompileSuccess OnMujocoCompileSuccess;

	UFUNCTION(BlueprintCallable, Category = "Mujoco")
	void SetActuatorValue(const FString& ActuatorName, double Value);

	UFUNCTION(BlueprintCallable, Category = "Mujoco")
	void SetActuatorValueByIndex(int32 ActuatorIndex, double Value);

	UFUNCTION(BlueprintCallable, BlueprintPure, Category = "Mujoco")
	double GetActuatorValue(const FString& ActuatorName) const;

	UFUNCTION(BlueprintCallable, BlueprintPure, Category = "Mujoco")
	double GetActuatorValueByIndex(int32 ActuatorIndex) const;

	UFUNCTION(BlueprintCallable, BlueprintPure, Category = "Mujoco")
	FVector2D GetActuatorRange(const FString& ActuatorName) const;

	UFUNCTION(BlueprintCallable, BlueprintPure, Category = "Mujoco")
	FVector2D GetActuatorRangeByIndex(int32 ActuatorIndex) const;

	UFUNCTION(BlueprintCallable, Category = "Mujoco")
	void SetJointValue(const FString& JointName, double Value);

	UFUNCTION(BlueprintCallable, Category = "Mujoco")
	void SetJointValueByIndex(int32 JointIndex, double Value);

	UFUNCTION(BlueprintCallable, BlueprintPure, Category = "Mujoco")
	double GetJointValue(const FString& JointName) const;

	UFUNCTION(BlueprintCallable, BlueprintPure, Category = "Mujoco")
	double GetJointValueByIndex(int32 JointIndex) const;

	
	virtual void PostRegisterAllComponents() override;	

	virtual void Tick(float DeltaTime) override;

	virtual void PostInitializeComponents() override;

protected:

	virtual void BeginPlay() override;
	virtual void EndPlay(const EEndPlayReason::Type EndPlayReason) override;
};
