#pragma once

#include "CoreMinimal.h"
#include "Components/SceneComponent.h"
#include "Structs/MujocoInertial.h"
#include "Misc/UniqueNamedComponent.h"
#include "MujocoBodyComponent.generated.h"

UCLASS(Blueprintable, ClassGroup = (Rendering, Common), hidecategories = (Object, Activation, "Components|Activation"), ShowCategories = (Mobility), editinlinenew, meta = (BlueprintSpawnableComponent))
class LUCKYMUJOCO_API UMujocoBodyComponent final : public USceneComponent, public TUniqueNamedComponent
{
	GENERATED_BODY()

public:
	UMujocoBodyComponent();

	UPROPERTY(Transient, VisibleAnywhere, Category = "Mujoco")
	int32 MujocoID = -1;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Body", meta = (Attribute = "mocap"))
	TOptional<bool> Mocap;

	/**
	 * Gravity compensation force, specified as fraction of body weight. This attribute creates an upwards force applied to the body’s center of mass, countering the force of gravity. As an example, a value of 1 creates an upward force equal to the body’s weight and compensates for gravity exactly. Values greater than 1 will create a net upwards force or buoyancy effect.
	 */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Body", meta = (Attribute = "gravcomp"))
	TOptional<double> GravComp;

	/**
	 * This element specifies the mass and inertial properties of the body. If this element is not included in a given body, the inertial properties are inferred from the geoms attached to the body.
	 */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Body")
	TOptional<FMujocoInertial> Inertial;

#if WITH_EDITORONLY_DATA
	virtual void OnComponentCreated() override;
	virtual void PostEditChangeProperty(FPropertyChangedEvent& PropertyChangedEvent) override;
#endif
};
