#pragma once

#include "CoreMinimal.h"
#include "Enums/MujocoEnums.h"
#include "MujocoGeom.generated.h"

USTRUCT(BlueprintType)
struct FMujocoGeom
{
	GENERATED_BODY()		

	/** 32-bit integer bitmasks used for contact filtering of dynamically generated contact pairs. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Geom", meta = (Attribute = "contype"))
	TOptional<int32> ConType;

	/** Bitmask for contact filtering. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Geom", meta = (Attribute = "conaffinity"))
	TOptional<int32> ConAffinity;

	/** Dimensionality of the contact space for a dynamically generated contact pair. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Geom", meta = (Attribute = "condim"))
	TOptional<EMujocoContactDim> ConDim;

	/** Integer group to which the geom belongs. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Geom", meta = (Attribute = "group"))
	TOptional<int32> Group;

	/** Geom priority determines how the properties of two colliding geoms are combined to form the properties of the contact. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Geom", meta = (Attribute = "priority"))
	TOptional<int32> Priority;

	/** Contact friction parameters for dynamically generated contact pairs. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Geom", meta = (Attribute = "friction"))
	TOptional<FMujocoFriction> Friction;

	/** Geom density is computed from the given mass. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Geom", meta = (Attribute = "mass"))
	TOptional<double> Mass;

	/** Material density used to compute the geom mass and inertia. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Geom", meta = (Attribute = "density"))
	TOptional<double> Density;

	/** If true, the geom’s inertia is computed assuming that all the mass is concentrated on the boundary. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Geom", meta = (Attribute = "shellinertia"))
	TOptional<bool> ShellInertia;

	/** Weight used for averaging of contact parameters. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Geom", meta = (Attribute = "solmix"))
	TOptional<double> Solmix;

	/** Constraint solver parameters for contact simulation. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Geom", meta = (Attribute = "solimp"))
	TOptional<FMujocoSolImp> SolImp;

	/** Constraint solver parameters for contact simulation. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Geom", meta = (Attribute = "solref"))
	TOptional<FMujocoSolRef> SolRef;

	/** Distance threshold below which contacts are detected and included in the global array mjData.contact. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Geom", meta = (Attribute = "margin"))
	TOptional<double> Margin;

	/** Enables the generation of inactive contacts. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Mujoco | Geom", meta = (Attribute = "gap"))
	TOptional<double> Gap;
};
