// Copyright 2022, Roberto De Ioris.

#pragma once

#include "CoreMinimal.h"
#include "Factories/Factory.h"
#include "Serialization/ArrayReader.h"
#include "Serialization/ArrayWriter.h"
#include "STLImportFactory.generated.h"

class UStaticMesh;

struct LUCKYMUJOCOEDITOR_API FUnrealSTLMesh
{
	TArray<FStaticMeshBuildVertex> Vertices;
	TArray<uint32> LODIndices;
	uint32 TrianglesNum;
	FBoxSphereBounds Bounds;
	TArray<FStaticMeshSection> Sections;

	FUnrealSTLMesh();
	FUnrealSTLMesh(const TArray<FUnrealSTLMesh>& Meshes);
};

UENUM()
enum class EUnrealSTLFileMode : uint8
{
	Auto,
	ASCII,
	Binary
};

USTRUCT(BlueprintType)
struct FUnrealSTLConfig
{
	GENERATED_BODY()

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "UnrealSTL")
	FTransform Transform;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "UnrealSTL")
	EUnrealSTLFileMode FileMode;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "UnrealSTL")
	bool bReverseWinding;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "UnrealSTL")
	UMaterialInterface* Material;

	FUnrealSTLConfig()
	{		
		Transform = FTransform::Identity;
		FileMode = EUnrealSTLFileMode::Auto;
		bReverseWinding = false;
		Material = nullptr;
	}
};

USTRUCT(BlueprintType)
struct FUnrealSTLStaticMeshConfig
{
	GENERATED_BODY()

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "UnrealSTL")
	bool bAllowCPUAccess;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "UnrealSTL")
	UObject* Outer;

	FUnrealSTLStaticMeshConfig()
	{
		bAllowCPUAccess = false;
		Outer = nullptr;
	}
};

USTRUCT(BlueprintType)
struct FUnrealSTLFile
{
	GENERATED_BODY()

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "UnrealSTL")
	FString Filename;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "UnrealSTL")
	FUnrealSTLConfig Config;
};

USTRUCT(BlueprintType)
struct FUnrealSTLFileLOD
{
	GENERATED_BODY()

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "UnrealSTL")
	TArray<FUnrealSTLFile> Sections;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "UnrealSTL")
	float ScreenSize;

	FUnrealSTLFileLOD() { ScreenSize = 0; }
};

/**
 *
 */
UCLASS()
class LUCKYMUJOCOEDITOR_API UUnrealSTLFactory : public UFactory
{
	GENERATED_BODY()

	bool LoadMeshFromSTLData(FArrayReader& Reader, const FUnrealSTLConfig& Config, FUnrealSTLMesh& STLMesh);
	UStaticMesh* LoadStaticMeshFromSTLFile(const FString& Filename, const FUnrealSTLConfig& Config, const FUnrealSTLStaticMeshConfig& StaticMeshConfig);
	UStaticMesh* LoadStaticMeshFromSTLFileLODs(const TArray<FUnrealSTLFileLOD>& FileLODs, const FUnrealSTLStaticMeshConfig& StaticMeshConfig);	

public:
	FUnrealSTLConfig ImportConfig;

	UUnrealSTLFactory();

	virtual UObject* FactoryCreateBinary(UClass* InClass, UObject* InParent, FName InName, EObjectFlags Flags, UObject* Context, const TCHAR* Type, const uint8*& Buffer, const uint8* BufferEnd, FFeedbackContext* Warn, bool& bOutOperationCanceled) override;
};